/*
 * HexaByteUTest.cpp
 *
 *  Created on: Jan 4, 2013
 *      Author: tuleu
 */

#include "HexaByteUTest.h"
#include <libsbcp/utils/HexaByte.h>
#include <sstream>
#include <stdexcept>
namespace sbcp {

TEST_F(HexaByteUTest,AnyAssignementInitializeValue){
	HexaByte a,b,c(0x03);
	EXPECT_FALSE(a.IsInitialized());
	EXPECT_FALSE(b.IsInitialized());
	//c is initialized by construction
	EXPECT_TRUE(c.IsInitialized());

	//a is initialized by assignation
	a = 0x00;
	EXPECT_TRUE(a.IsInitialized());

	//b is assigned by parsing
	std::istringstream is("0xae");
	is >> b;
	EXPECT_TRUE(b.IsInitialized());
}


TEST_F(HexaByteUTest,CanBeOutputtedToStream){
	std::ostringstream os;
	HexaByte a(0xef);
	os << a;
	EXPECT_EQ("0xef",os.str());
}

TEST_F(HexaByteUTest,CouldBeParsedFromStream){
	//two foramt are available : w/o 0x prefix
	std::istringstream is("eb 0xec");
	HexaByte eb,ec;
	is >> eb >> ec;
	EXPECT_EQ(0xeb,eb.AsByte());
	EXPECT_EQ(0xec,ec.AsByte());

}

TEST_F(HexaByteUTest,UnitializedValueIsZero){
	HexaByte uninit;
	EXPECT_FALSE(uninit.IsInitialized());
	EXPECT_EQ(0,uninit.AsByte());
}

TEST_F(HexaByteUTest,OverflowParsingThrowOutOfRangeException){
	std::istringstream is("0xffff");//overflow a 8-bit bytes
	HexaByte overflow;
	EXPECT_THROW({is >> overflow;},std::out_of_range);
}

} /* namespace sbcp */
